<?php
/*
* Class Generator Class: una jararquﻎa de clases para generar cﻩdigo de clases en PHP 5.
* versiﻩn 0.4
*
* @Prometeus Technology, a subsidiary of Pegasus Tech Supply
* @copyright Copyright (C) 2008-2010 Leonardo Tadei. All rights reserved.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see gpl-3.0.txt
* Class Generator is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
*/

/*
* Generador de cﻩdigo PHP5 para construir clases
*/

class genClass {

   // Atributos
   private $name;
   private $inherit;
   private $interfaces;
   private $comment;
   public $attributes = NULL;
   public $methods = NULL;

   // Metodos
   public function __construct ($nom, $com='', $her='', $int='') {
      $this->attributes = new genAttributeCollection();
      $this->methods = new genMethodCollection();
      $this->setName($nom);
		$this->setComment($com);
		$this->setInherit($her);
		$this->setInterfaces($int);
		// Siempre tiene un Constructor
		$st = new genMethod("__construct");
		$this->methods->add($st);
   }

   public function toCode () {
		$tmp = $this->formatComment($this->getComment());
		$tmp .= "class {$this->getName()}";
		$tmp .= $this->getInherit() == '' ? '' : " extends {$this->getInherit()}";
		$tmp .= $this->getInterfaces() == '' ? '' : " implements {$this->getInterfaces()}";
		$tmp .= " {\n";
		$tmp .= "\n\t/*\n\t* Atributos\n\t*/\n";
		// Crea una colección para cargar los setters, getters y toString
      $setget = new genMethodCollection();
		// Pasa a código los atributos
		try {
			foreach ($this->attributes as $l) {
				$tmp .= "\t".$l->toCode();

				$st = new genMethod("set".UcFirst($l->getName()),'$v');
				$st->setImplementation("\$this->{$l->getName()} = \$v;");
				$setget->add( $st );
				//$st = new genMethod("get".UcFirst($l->getName())."()");
				$st = new genMethod("get".UcFirst($l->getName()));
				$st->setImplementation("return \$this->{$l->getName()};");
				$setget->add( $st );
			}
		} catch (Exception $e) {
			// atrapa el error que se genera si no se definen atributos
		}
		$tmp .= "\n\t/*\n\t* Metodos\n\t*/\n";
		// Pasa a cﻩdigo los mﻯtodos agregados
		foreach ($this->methods as $l) {
			$tmp .= "\t".$l->toCode();
		}
		$tmp .= "\n\t/*\n\t* Setters y Getters\n\t*/\n";

		// Agrega al final un toString
		$st = new genMethod("__toString");
		$st->setImplementation("return \"class {$this->getName()}\";");
		$setget->add( $st );
		// Pasa a cﻩdigo los mﻯtodos set y get agregados
		foreach ($setget as $l) {
			$tmp .= "\t".$l->toCode();
		}

		return $tmp .= "\n}\t// class {$this->getName()}";
   }

	private function formatComment ($v) {
		// Devuelve un comentario formateado como cﻩdigo
		if ($v == '') { return ''; }
		$a = explode("\n", $v);
		$tmp = "/* ***\n";
		foreach($a as $l){
			$tmp .= "* $l\n";
		}
		return $tmp .= "*** */\n";
	}

   // Setters y Getters
   public function setName ($v) {
   	// El nombre es obligatorio
   	if($v == '') {
   		throw new Exception("Name must be defined: $v");
		}
      $this->name = $v;
   }
   public function getName () {
      return $this->name;
   }

   public function setInherit ($v) {
      $this->inherit = $v;
   }
   public function getInherit () {
      return $this->inherit;
   }

   public function setInterfaces ($v) {
      $this->interfaces = $v;
   }
   public function getInterfaces () {
      return $this->interfaces;
   }

   public function setComment ($v) {
      $this->comment = $v;
   }
   public function getComment () {
      return $this->comment;
   }

} // Class GenClass


/*
* Atributos de una clase
*/

class genAttribute {

   // Atributos
   private $visibility = "private";
   private $name;
   private $initValue;
   private $comment;

   // Metodos
   public function __construct ($name, $ival='', $comm='', $vis='private') {
      $this->setVisibility($vis);
      $this->setName($name);
      $this->setInitValue($ival);
      $this->setComment($comm);
   }

   public function toCode () {
   	$fiv = '';
   	$fcom = '';
   	if($this->getInitValue() != '') { $fiv = " = {$this->getInitValue()}"; }
   	if($this->getComment() != '') { $fcom = " // {$this->getComment()}"; }
		return "{$this->getVisibility()} \${$this->getName()}$fiv;\t$fcom\n";
   }

   // Setters y Getters
   public function setVisibility ($v) {
   	// valida la visibilidad
   	if (!(($v == 'private') || ($v == 'protected') || ($v == 'public'))) {
   		throw new Exception("Visibility invalid: $v");
   	}
      $this->visibility = $v;
   }
   public function getVisibility () {
      return $this->visibility;
   }

   public function setName ($v) {
   	// El nombre es obligatorio
   	if($v == '') {
   		throw new Exception("Name must be defined: $v");
		}
      $this->name = $v;
   }
   public function getName () {
      return $this->name;
   }

   public function setInitValue ($v) {
      $this->initValue = $v;
   }
   public function getInitValue () {
      return $this->initValue;
   }

   public function setComment ($v) {
      $this->comment = $v;
   }
   public function getComment () {
      return $this->comment;
   }

   public function __toString () {
      return "clase: GenAttributes";
   }

} // Class GenAttributes

/*
* Métodos de una clase
*/

class genMethod {

   // Atributos
   private $visibility = 'public';
   private $name;
   private $message;
   private $comment;
   private $implementation;

   // Metodos
   public function __construct ($name, $msg='', $comm='', $vis='public', $imp='') {
      $this->setVisibility($vis);
      $this->setName($name);
      $this->setMessage($msg);
      $this->setComment($comm);
      $this->setImplementation($imp);

   }

   public function toCode () {
   	$fcom = '';
   	$fimp = '';
   	if($this->getMessage() != '') { $fmsg = "({$this->getMessage()})"; } else { $fmsg = "()"; }
   	if($this->getComment() != '') { $fcom = "// {$this->getComment()}"; }
//   	if($this->getImplementation() != '') { $fimp = str_replace("\n","\n\t\t",$this->getImplementation()); }
   	if($this->getImplementation() != '') { $fimp = str_replace("\n","\t\t",$this->getImplementation()); }
		return "{$this->getVisibility()} function {$this->getName()}$fmsg {
		$fcom$fimp
	}\n";
   }

   // Setters y Getters
   public function setVisibility ($v) {
   	// valida la visibilidad
   	if (!(($v == 'private') || ($v == 'protected') || ($v == 'public'))) {
   		throw new Exception("Visibility invalid: $v");
   	}
      $this->visibility = $v;
   }
   public function getVisibility () {
      return $this->visibility;
   }

   public function setName ($v) {
   	// El nombre es obligatorio
   	if($v == '') {
   		throw new Exception("Name must be defined: $v");
		}
      $this->name = $v;
   }
   public function getName () {
      return $this->name;
   }

   public function setMessage ($v) {
      $this->message = $v;
   }
   public function getMessage () {
      return $this->message;
   }

   public function setComment ($v) {
      $this->comment = $v;
   }
   public function getComment () {
      return $this->comment;
   }

   public function setImplementation ($v) {
      $this->implementation = $v;
   }
   public function getImplementation () {
      return $this->implementation;
   }

   public function __toString () {
      return "clase: GenMethods";
   }

} // Class GenMethods

/*
* Implementa una Colección genérica para especializar
*/
abstract class genCollection implements IteratorAggregate {
	protected $items = Array();

	// Métodos
	public function getIterator() {
		return new ArrayIterator($this->items);
	}

//	public function add(genCollection $obj) {
	public function add($obj) {
		$this->items[] = $obj;
	}
} // class Collection

/*
* Coleccion de Atributos
*/
class genAttributeCollection extends genCollection {
	// Métodos
//	public function add(genAttribute $obj) {
//	public function add(genCollection $obj) {
	public function add( $obj) {
		$this->items[] = $obj;
	}
}

/*
* Coleccion de Métodos
*/
class genMethodCollection extends genCollection {
	// Métodos
//	public function add(genMethod $obj) {
//	public function add(genCollection $obj) {
	public function add( $obj) {
		$this->items[] = $obj;
	}
}

?>
