<?php
/*
* @Facultad de Ingeniería, Universidad Nacional de Mar del Plata
* @copyright Copyright (C) 2015 Leonardo Tadei. All rights reserved.
* @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see gpl-3.0.txt
* Panel de Estado is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
*/

require_once('class.Estado.php');

class Servicio {
	/*
	* Atributos
	*/
	private $id;
	private $nom;
	private $estado = null;	 // Instancia de Estado
	private $des;
	private $default;
	/*
	* Métodos
	*/
	public function __construct($nom='', Estado $est=null, $des='', $default='') {
		$this->setNom($nom);
		$this->setEstado($est);
		$this->setDes($des);
		$this->setDefault($default);
	}
	/*
	* Setters y Getters
	*/
	public function setId($v) {
        // Setter de un solo uso
        if ($this->id == null) {
            $this->id = $v;
        } else throw new Exception("Servicio: el id no puede ser cambiado");
	}
	public function getId() {
		return $this->id;
	}
	public function setNom($v) {
		$this->nom = $v;
	}
	public function getNom() {
		return $this->nom;
	}
	public function setEstado($v) {
		$this->estado = $v;
	}
	public function getEstado() {
		return $this->estado;
	}
	public function setDes($v) {
		$this->des = $v;
	}
	public function getDes() {
		// Devuelve la Descripción dependiendo del Estado
		if( $this->getEstado()->getId() == 1 ) {
			return $this->getDefault();
		} else {
			return $this->des;	// Poner acá el getter genera una llamada recursiva
		}
	}
	public function setDefault($v) {
		$this->default = $v;
	}
	public function getDefault() {
		return $this->default;
	}
	public function __toString() {
		return "class Servicio";
	}
}	// class Servicio

class CollectionServicio implements IteratorAggregate {
    private $items = array();
    // Métodos
    public function getIterator() {
        return new ArrayIterator($this->items);
    }
    public function add(Servicio $obj) {
        $this->items[] = $obj;
    }
    public function getById($id) {
    	// Devuelve un objeto por su ID o falso
    	foreach($this->items as $o){
			if($id==$o->getId()) {
				return $o;
			}    	
    	}
    	return null;
    }
}  // class CollectionServicio